#include <LiquidCrystal.h>
LiquidCrystal lcd(2, 3, 4, 5, 6, 7);

void setup() {
  Serial.begin(115200);
  lcd.begin(16,2);
  lcd.clear();
  lcd.print("* LCD Smartie *");
  lcd.setCursor(0,1);
  lcd.print("Franzis Arduino ");
}

byte serial_getch() {
  int incoming;
  while (Serial.available()==0) {}
  incoming = Serial.read(); 
  return (byte) (incoming &0xff);
}

void loop() {
  byte rxbyte;
  byte temp;
  byte col;
  byte row;
  uint8_t location;
  uint8_t charMap[7];

  rxbyte = serial_getch();
  if (rxbyte == 254) //Matrix Orbital verwendet 254 Präfix für Kommandos
  {
    switch (serial_getch())
    {
      case 71:  //Cursorposition setzen
        col = (serial_getch() - 1);
        row = (serial_getch());
        lcd.setCursor(col,row-1);
        break;
      case 72:  //Cursor Home
        lcd.setCursor(0,0);
        break;
      case 78:  //Benutzerdefinierte Zeichen
        location = serial_getch();
        for (temp = 0; temp < 8; temp++)
        {
          charMap[temp] = serial_getch(); //Bytemuster
        }
        lcd.createChar(location, charMap);
        break;
      case 88:  //Display löschen
        lcd.clear();
        lcd.setCursor(0,0);
        break;

      //Diese Kommandos werden ignoriert
      case 35: //read serial number
      case 36: //read version number
      case 55: //read module type
      case 59: //exit flow-control mode
      case 65: //auto transmit keypresses
      case 66: //backlight on
      case 96: //auto-repeat mode off (keypad)
      case 67: //auto line-wrap on
      case 68: //auto line-wrap off
      case 70: //backlight off
      case 74: //show underline cursor
      case 75: //underline cursor off
      case 76: //move cursor left
      case 77: //move cursor right
      case 81: //auto scroll on
      case 82: //auto scroll off
      case 83: //show blinking block cursor
      case 84: //block cursor off
      case 86: //GPO OFF
      case 87: //GPO ON
      case 104: //init horiz bar graph
      case 109: //init med size digits
      case 115: //init narrow vert bar graph
      case 118: //init wide vert bar graph
      case 152: //set and remember
      case 153: //set backlight brightness
        break;
      default:
        //alle anderen Kommandos werden ignoriert und das Parameter-Byte gelöscht
        temp = serial_getch();
        break;
    }
    return;
  } //ENDE DES KOMMANDO-HANDLERS

  switch (rxbyte)
  {
    //Zeichen, die im Zeichensatz enthalten sind
    case 0xE4: //ASCII "a" Umlaut
      rxbyte = 0xE1;
      break;
    case 0xF1: //ASCII "n" Tilde
      rxbyte = 0xEE;
      break;
    case 0xF6: //ASCII "o" Umlaut
      rxbyte = 0xEF; 
      break;
    case 0xFC: //ASCII "u" Umlaut
      rxbyte = 0xF5;
      break;

    //Akzent -> Normaler Buchstabe ...und weitere Symbolumsetzungen
    case 0xA3: //Pfund Sterling 
      rxbyte = 0xED;
      break;
    case 0xB5: //My
      rxbyte = 0xE4;
      break;
    case 0xC0: //"A" Varianten
    case 0xC1:
    case 0xC2:
    case 0xC3:
    case 0xC4:
    case 0xC5:
      rxbyte = 0x41;
      break;
    case 0xC8: //"E" Varianten
    case 0xC9:
    case 0xCA:
    case 0xCB:
      rxbyte = 0x45;
      break;
    case 0xCC: //"I" Varianten
    case 0xCD:
    case 0xCE:
    case 0xCF:
      rxbyte = 0x49;
      break;
    case 0xD1: //"N" Tilde -> "N"
      rxbyte = 0x43;
      break;
    case 0xD2: //"O" Varianten
    case 0xD3:
    case 0xD4:
    case 0xD5:
    case 0xD6:
    case 0xD8:
      rxbyte = 0x4F;
      break;
    case 0xD9: //"U" Varianten
    case 0xDA:
    case 0xDB:
    case 0xDC:
      rxbyte = 0x55;
      break;
    case 0xDD: //"Y" Acute -> "Y"
      rxbyte = 0x59;
      break;
    case 0xE0: //"a" Varianten außer Umlaut
    case 0xE1:
    case 0xE2:
    case 0xE3:
    case 0xE5:
      rxbyte = 0x61;
      break;
    case 0xE7: //"c" Cedilla -> "c"
      rxbyte = 0x63;
      break;
    case 0xE8: //"e" Varianten
    case 0xE9:
    case 0xEA:
    case 0xEB:
      rxbyte = 0x65;
      break;
    case 0xEC: //"i" Varianten
    case 0xED:
    case 0xEE:
    case 0xEF:
      rxbyte = 0x69;
      break;
    case 0xF2: //"o" Varianten außer Umlaut
    case 0xF3:
    case 0xF4:
    case 0xF5:
    case 0xF8:
      rxbyte = 0x6F;
      break;
    case 0xF7: //Divisionssymbol
      rxbyte = 0xFD;
      break;
    case 0xF9: //"u" Varianten außer Umlaut
    case 0xFA:
    case 0xFB:
      rxbyte = 0x75;
      break;
    default:
      break;
  }

  lcd.write(rxbyte);
}

