/*
  TextDisplay.cpp - Buffered alphanumeric display for a ST7565 based 12864 LCD.
 Thomas Baum, th.baum@online.de, 12.09.2014
 Version MEGA, LEONARDO
 */

#include "TextDisplay.h"
#include "Display.h"
#include "Font.h"

void TextDisplay::init(byte contrast) {
  lcd = Display();
  lcd.init(contrast);
  lcd.clearDisplayRAM();
  lcd.setPageAddress(0);
  lcd.setColumnAddress(0);

  curserPositionY = 0;
  curserPositionX = 0;
  textBuffer[MAX_CHARACTERS_Y][MAX_CHARACTERS_X];
}

void TextDisplay::println(const char* string) {
  int position = 0;
  while (string[position]) {
    char character = string[position];
    print(character);
    string++;
  }
  print(13);
}

void TextDisplay::clear(void) {
  print(255);
}

void TextDisplay::print(char character) {
  switch (character) {

  case 255:
    for (byte y=0; y<MAX_CHARACTERS_Y; y++) {
      for (byte x=0; x<MAX_CHARACTERS_X; x++) {
        textBuffer[y][x] = 0;
      }
    }
    writeTextBuffer();
    curserPositionY = 0;
    curserPositionX = 0;
    break;

  case 127:  //Delete (Putty Backspace)
    if (curserPositionX == 0 && curserPositionY > 0) {
      curserPositionY--;
      curserPositionX = getLastCharPosition(curserPositionY);
      textBuffer[curserPositionY][curserPositionX] = 0; 
    } 
    else if (curserPositionX > 0) {
      curserPositionX--;
      textBuffer[curserPositionY][curserPositionX] = 0;
    } 
    else {
    }
    break;

  case 13:  //CR
    if (curserPositionY < MAX_CHARACTERS_Y-1) {
      curserPositionY++;
      curserPositionX = 0;
    } 
    break;

  default:
    if (curserPositionX < MAX_CHARACTERS_X) {
      textBuffer[curserPositionY][curserPositionX] = character;
      curserPositionX++;
    } 
    else if (curserPositionY < MAX_CHARACTERS_Y-1) {
      curserPositionY++;
      curserPositionX = 0;
      textBuffer[curserPositionY][curserPositionX] = character;
      curserPositionX++;
    }
    break;
  }
  writeTextBuffer();
} 

byte TextDisplay::getLastCharPosition(byte line) {
  for (byte x=0; x<MAX_CHARACTERS_X; x++) {
    if(textBuffer[line][x] == 0) {
      return (x!=0) ? x : 0;
    }
  }
  return MAX_CHARACTERS_X-1;
}

void TextDisplay::writeTextBuffer() {
  for (byte y=0; y<MAX_CHARACTERS_Y; y++) {
    lcd.setPageAddress(y);
    lcd.setColumnAddress(0);
    for (byte x=0; x<MAX_CHARACTERS_X; x++) {
      int position = textBuffer[y][x] * CHARACTER_WIDTH;
      for (byte i=0; i<CHARACTER_WIDTH; i++) {
        lcd.writeData(pgm_read_byte(font7x5 + position++));
      }
    }
  }
}


