/*
  Turtle.c - Minimal implementation of Turtlegraphics(logo) for a ST7565 based 12864 LCD.
  Created by Thomas Baum, th.baum@online.de, 26.11.2014
*/

#include <util/delay.h>
#include <SPI.h>
#include "Turtle.h"
#include "Display.h"

Display lcd = Display();
word heading = 0;
int ox=63, oy=31, nx, ny;
boolean isPenDown = true;

void Turtle::init(byte contrast) {
  lcd.init(contrast);
  lcd.clearVideoBuffer();
}

void Turtle::forward(byte distance) {
  float rad = (heading + 180) / 57.2957795;

  nx = ox + (int)(distance * sin(rad));
  ny = oy + (int)(distance * cos(rad));
  if (isPenDown) {
    lcd.drawLine(ox,oy, nx, ny);
  }
  
  ox = nx;
  oy = ny;
  lcd.show();
}

void Turtle::back(byte distance) {
  float rad = heading / 57.2957795;

  nx = ox + (int)(distance * sin(rad));
  ny = oy + (int)(distance * cos(rad));
  if (isPenDown) {
    lcd.drawLine(ox,oy, nx, ny);
  }
  ox = nx;
  oy = ny;
  lcd.show();
}

void Turtle::right(byte angle) {
  heading = (heading + 360 - angle) % 360;
}

void Turtle::left(byte angle) {
  heading = (heading + 360 + angle) % 360;
}

void Turtle::home() {
  ox = 63;
  oy = 31;
}

void Turtle::clean() {
  lcd.clearVideoBuffer();
  lcd.show();
}

void Turtle::cleanscreen() {
  clean();
  home();
}

void Turtle::penup() {
  isPenDown = false;
}

void Turtle::pendown() {
  isPenDown = true;
}

